﻿
using System;
using System.Windows;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace VZInterfaceTestHandbook
{
  /**
   * Wolfprot commands for authentication
   */
  public partial class Authentication
  {
    IComIf comIf;
    /**
     * @param com: uses Interface IComIf
     */
    public Authentication(IComIf com)
    {
      comIf = com;
    }

    /**
     * @return 4 byte session ID random number
     */
    //@@@START_REQUESTSESSIONID@@@
    public Byte[] Request_SessionID()
    {
      /*Declaration*/
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[7];
      Byte[] sessionID = new Byte[4];
      Boolean iReturn;

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xD3;
      bufSnd[2] = 0x00;

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return null;
      }

      else
      {
        // Take the session ID out of the 
        // received bytearray
        for (int i = 0; i < sessionID.Length; i++)
        {
          sessionID[i] = bufRecv[i + 3];
        }
      }

      return sessionID;
    }
    //@@@STOP_REQUESTSESSIONID@@@

    /**
     * Login command
     * @param username: admin or guest
     * @param password: password max 10 characters
     * @param ip: IP address
     * @return success = false
     */
    //@@@START_LOGIN@@@
    public Boolean Login(Byte[] username, Byte[] password, Byte[] ip)
    {
      /*Declaration*/
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[39];
      Byte[] bufRecv = new Byte[3];
      Byte[] sessionID;
      Byte[] md5Out;
      Boolean iReturn;

      /*Send command 0xD3*/
      sessionID = Request_SessionID();
      if (sessionID == null)
        return true;

      /*Calc MD5*/
      ArrayList aList = new ArrayList();

      // Create the array list for the MD5 Hash Code
      aList.AddRange(sessionID);
      aList.AddRange(username);
      aList.AddRange(password);
      aList.AddRange(ip);

      // Create the Hash Code
      System.Security.Cryptography.MD5CryptoServiceProvider x = new System.Security.Cryptography.MD5CryptoServiceProvider();
      md5Out = x.ComputeHash((Byte[])aList.ToArray(typeof(Byte)));

      // Clear Arraylist
      aList.Clear();
      // Initialising the command   
      aList.AddRange(new Byte[] { 0x01, 0xD4, 0x24 });
      // Initialising the md5 to the command   
      aList.AddRange(md5Out);
      // Initialising the username to the command          
      aList.AddRange(username);
      // Initialising the IP Adress to the command            
      aList.AddRange(ip);

      bufSnd = (Byte[])aList.ToArray(typeof(Byte));

      /*Send command 0xD4*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return true;
      }
      else
        MessageBox.Show("Login successful");

      return false;
    }
    //@@@STOP_LOGIN@@@

    /**
     * Logout command
     * @return success = false
     */
    //@@@START_LOGOUT@@@
    public Boolean Logout()
    {
      /*Declaration*/
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[3];
      Boolean iReturn;

      /*Assignment*/
      bufSnd[0] = 0x01;
      bufSnd[1] = 0xD5;
      bufSnd[2] = 0x01;
      bufSnd[3] = 0x00;

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return true;
      }
      else
        MessageBox.Show("Logout successful");

      return false;
    }
    //@@@STOP_LOGOUT@@@
  }
}
