﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace VZInterfaceTestHandbook
{
  public partial class FormAnyCommand : Form
  {
    IComIf comIf;
    Boolean isConnected = false;

    public FormAnyCommand(IComIf com)
    {
      comIf = com;
      InitializeComponent();
    }

    /**
     * Open connection
     * @return success = false
     */
    private Boolean connOpen()
    {
      if (comIf.Open())
      {
        MessageBox.Show("Couldn´t open interface!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      isConnected = true;
      return false;
    }

    /**
     * Close connection
     */
    private void connClose()
    {
      if (isConnected)
        comIf.Close();
      isConnected = false;
    }

    private void buttonSend_Click(object sender, EventArgs e)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      String strText = textBoxRequest.Text;
      Byte[] bufSnd = createPacket(strText);
      Byte[] bufRecv = new Byte[1024];
      Boolean iReturn;

      if (connOpen())
        return;
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      connClose();
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return;
        }
      }

      StringBuilder strBuf = new StringBuilder();
      for (int i = 0; i < bufRecv.Length; i++)
      {
        strBuf.Append(convertBin2Ascii(bufRecv[i]));
      }
      strBuf.Append("\n");
      textBoxReply.AppendText(strBuf.ToString());
    }

    /**
     * support Enter
     */
    private void textBoxRequest_KeyPress(object sender, KeyPressEventArgs e)
    {
      if (e.KeyChar == 13)
      {
        buttonSend_Click(sender, e);
      }
    }

    /**
     * Create packet
     * @param strText: text from textbox
     * @return strText as binary byte array
     */
    private Byte[] createPacket(String strText)
    {
      if (strText == null)
      {
        MessageBox.Show("Command is empty!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return (null);
      }
      strText = strText.Replace(" ", ""); //strip spaces
      if ((strText.Length % 2) != 0)
      {
        MessageBox.Show("Length must be multiple of 2!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return (null);
      }
      Byte[] bBufText = new Byte[strText.Length];
      Byte[] bBufOut = new Byte[strText.Length / 2];
      System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
      bBufText = enc.GetBytes(strText);
      for (int i = 0; i < strText.Length; i++)
      {
        if (((i + 1) % 2) == 0)
        {
          bBufOut[i / 2] = (Byte)((makeNibble(bBufText[i - 1]) << 4) | (makeNibble(bBufText[i])));
        }
      }
      //for (int i = 0; i < bBufOut.Length; i++)
      //  Console.WriteLine("o=" + bBufOut[i]);
      return (bBufOut);
    }

    /**
     * Converting from ASCII to Bin
     */
    private Byte makeNibble(Byte bByte)
    {
      switch ((char)bByte)
      {
        case 'a':
        case 'A':
          return (0x0a);
        case 'b':
        case 'B':
          return (0x0b);
        case 'c':
        case 'C':
          return (0x0c);
        case 'd':
        case 'D':
          return (0x0d);
        case 'e':
        case 'E':
          return (0x0e);
        case 'f':
        case 'F':
          return (0x0f);
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
          return ((Byte)(bByte - 0x30));
      }
      Console.WriteLine("Parse Error: Character not allowed!" + bByte);
      return (0);
    }

    /**
     * Convert from Bin to 2 Byte ASCII String
     */
    private String convertBin2Ascii(Byte bByte)
    {
      Byte bHiByte, bLoByte;
      Byte[] bBuf = new Byte[2];

      bHiByte = (Byte)((bByte >> 4) & 0x0F);
      switch (bHiByte)
      {
        case 0x0f:
        case 0x0e:
        case 0x0d:
        case 0x0c:
        case 0x0b:
        case 0x0a:
          bBuf[0] = (Byte)(bHiByte + 87);
          break;
        default:
          bBuf[0] = (Byte)(bHiByte + 0x30);
          break;
      }
      bLoByte = (Byte)(bByte & 0x0F);
      switch (bLoByte)
      {
        case 0x0f:
        case 0x0e:
        case 0x0d:
        case 0x0c:
        case 0x0b:
        case 0x0a:
          bBuf[1] = (Byte)(bLoByte + 87);
          break;
        default:
          bBuf[1] = (Byte)(bLoByte + 0x30);
          break;
      }
      System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
      return (enc.GetString(bBuf));
    }

  }
}
