﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace VZInterfaceTestHandbook
{
  public partial class FormFirmwareUpdate : Form
  {
    int step = 0;
    IComIf comIf;
    int maxPacketLength;
    Byte[] fWData;
    Boolean isConnected = false;

    /**
     * Initialize the form
     * @param com: uses Interface IComIf
     * @param iPacketLen: max packet length
     */
    public FormFirmwareUpdate(IComIf com, int iPacketLen)
    {
      maxPacketLength = iPacketLen;
      comIf = com;
      InitializeComponent();
    }

    /**
     * Open connection
     * @return success = false
     */
    private Boolean connOpen()
    {
      if (comIf.Open())
      {
        MessageBox.Show("Couldn´t open interface!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      isConnected = true;
      return false;
    }

    /**
     * Close connection
     */
    private void connClose()
    {
      if (isConnected)
        comIf.Close();
      isConnected = false;
    }

    private void buttonReturn_Click(object sender, EventArgs e)
    {
      connClose();
      Close();
    }

    private void buttonSearch_Click(object sender, EventArgs e)
    {
      OpenFileDialog selecter = new OpenFileDialog();
      if (selecter.ShowDialog() == DialogResult.OK)
      {
        textBoxPath.Text = selecter.FileName.ToString();
        buttonUpdate.Enabled = true;
      }
    }

    private void timer1_Tick(object sender, EventArgs e)
    {
      if (textBoxPath.Text.EndsWith(".w19"))
      {
        StateMachineFb4Eye12();
      }
      else if (textBoxPath.Text.EndsWith(".wgz"))
      {
        StateMachinePf1();
      }
      else
      {
        timerFWUpdate.Stop();
        connClose();
        MessageBox.Show("Unsupported file!");
      }
    }

    /**
     * FB4, EYE12 state machine
     * Order of commands:
     * 1) Buildnumber
     * 2) (Firmware Downgrade)
     * 3) Firmware Upload Erase Flash Set
     * 4) Firmware Upload Erase Flash Get
     * 5) Firmware Upload Data Start
     * 6) Firmware Upload Data
     * 7) Firmware Upload Data Stop
     */
    //@@@START_STATEMACHINE_FB4@@@
    public void StateMachineFb4Eye12()
    {
      int iUpDowngrade = 0;

      // Enable the commands
      FirmwareUpdate fWUpdate = new FirmwareUpdate(comIf);

      /*step0*/
      if (step == 0)
      {
        if (FWUpOrDowngradeControl(ref iUpDowngrade))
        {
          timerFWUpdate.Stop();
          connClose();
        }
        switch (iUpDowngrade)
        {
          case 0:
            step = 0;
            timerFWUpdate.Stop();
            connClose();
            break;
          case -1:  //Downgrade
            step++;
            break;
          case 1:   //Upgrade
            step += 2;
            break;
        }
      }
      // Enable downgrade if needed
      /*step1*/
      else if (step == 1)
      {
        // Increment step to go to next command
        step++;

        // Initialize the method to erase the flash
        // and stop the timer in case of an error
        if (fWUpdate.FWDowngrade())
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Start the erase flash
      /*step2*/
      else if (step == 2)
      {
        // Increment step to go to next command
        step++;

        // Initialize the method to erase the flash
        // and stop the timer in case of an error
        if (fWUpdate.UploadEraseFlash())
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Get the state of erasing flash
      // to know when it's finished
      /*step3*/
      else if (step == 3)
      {
        // Change the text of the label
        labelUpdateState.Text = "Update: Erasing ... "
                                + fWUpdate.GetUploadEraseFlash().ToString() + "%";

        /* Change the value of the progress bar
         * and initialize the method for getting the
         * state of erasing flash if the received is
         * not an error number (200 choosen)
         */
        if (fWUpdate.GetUploadEraseFlash() > 100)
        {
          timerFWUpdate.Stop();
          connClose();
        }
        else
          progressBar1.Value = fWUpdate.GetUploadEraseFlash();

        if (progressBar1.Value == 100)
        {
          // Increment step to go to next command
          step++;

          // Change the text of the label
          labelUpdateState.Text = "Update: Erasing ... "
                                  + fWUpdate.GetUploadEraseFlash().ToString() + "%";
        }
      }
      // Start the upload of the new firmware
      /*step4*/
      else if (step == 4)
      {
        // Increment step to go to the next command
        step++;

        Byte[] fWDataLength = new Byte[4];
        fWDataLength[0] = Convert.ToByte(fWData.Length >> 24);
        fWDataLength[1] = Convert.ToByte((fWData.Length & 0x00FF0000) >> 16);
        fWDataLength[2] = Convert.ToByte((fWData.Length & 0x0000FF00) >> 8);
        fWDataLength[3] = Convert.ToByte(fWData.Length & 0x000000FF);

        // Initialize the method to start new firmware
        if (fWUpdate.UploadDataStart(fWDataLength))
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Upload the new firmware
      /*step5*/
      else if (step == 5)
      {
        // Increment step to go to the next command
        step++;
        /* Initialize the method to generate the needed information
         * to upload the new firmware
         */
        if (Upload("Programming"))
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Stop/finish the firmware update
      /*step6*/
      else if (step == 6)
      {
        // Change the text of the label
        labelUpdateState.Text = "Update: Finished";

        // Stop the timer
        timerFWUpdate.Stop();

        // Reset the step
        step = 0;

        // Initialize the method to stop the firmware update
        fWUpdate.UploadStop();
        /*Close interface*/
        connClose();
        // Show a message box to inform that the 
        // firmware update has finished
        MessageBox.Show("Update finished." +
                       "\nThe visualizer restarts in 30 - 150 seconds.");
      }
    }
    //@@@STOP_STATEMACHINE_FB4@@@

    /*
     * PF1 state machine
     * Order of commands:
     * 1) Buildnumber
     * 2) Firmware Upload Data Start
     * 3) Firmware Upload Data
     * 4) Firmware Upload Data Stop
     * 5) Firmware Upload Erase Flash Set
     * 6) Firmware Upload Erase Flash Get
     */
    //@@@START_STATEMACHINE_PF1@@@
    public void StateMachinePf1()
    {
      int iUpDowngrade = 0;
      // Enable the commands
      FirmwareUpdate fWUpdate = new FirmwareUpdate(comIf);

      // Initialize control if down or upgrade
      /*step0*/
      if (step == 0)
      {
        if (FWUpOrDowngradeControl(ref iUpDowngrade))
        {
          timerFWUpdate.Stop();
          connClose();
        }
        switch (iUpDowngrade)
        {
          case 0:
            step = 0;
            timerFWUpdate.Stop();
            connClose();
            break;
          case -1:  // Downgrade there is no downgrade command in PF1
          case 1:   // Upgrade
            step++;
            break;
        }
      }
      /*step1*/
      else if (step == 1)
      {
        // Increment step to go to the next command
        step++;

        Byte[] fWDataLength = new Byte[4];
        fWDataLength[0] = Convert.ToByte(fWData.Length >> 24);
        fWDataLength[1] = Convert.ToByte((fWData.Length & 0x00FF0000) >> 16);
        fWDataLength[2] = Convert.ToByte((fWData.Length & 0x0000FF00) >> 8);
        fWDataLength[3] = Convert.ToByte(fWData.Length & 0x000000FF);

        // Initialize the method to start new firmware
        if (fWUpdate.UploadDataStart(fWDataLength))
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Upload the new firmware
      /*step2*/
      else if (step == 2)
      {
        // Increment step to go to the next command
        step++;
        /* Initialize the method to generate the needed information
         * to upload the new firmware
         */
        if (Upload("Loading"))
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Stop/finish the firmware update
      /*step3*/
      else if (step == 3)
      {
        step++;

        // Initialize the method to stop the firmware update
        if (fWUpdate.UploadStop())
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Start the programming
      /*step4*/
      else if (step == 4)
      {
        // Increment step to go to next command
        step++;

        // Initialize the method to erase the flash
        // and stop the timer in case of an error
        if (fWUpdate.UploadEraseFlash())
        {
          timerFWUpdate.Stop();
          connClose();
        }
      }
      // Get the state of programming
      /*step5*/
      else if (step == 5)
      {
        // Change the text of the label
        labelUpdateState.Text = "Update: Programming ... "
                                + fWUpdate.GetUploadEraseFlash().ToString() + "%";

        /* Change the value of the progress bar
         * and initialize the method for getting the
         * state of erasing flash if the received is
         * not an error number (200 choosen)
         */
        if (fWUpdate.GetUploadEraseFlash() > 100)
        {
          timerFWUpdate.Stop();
          connClose();
        }
        else
          progressBar1.Value = fWUpdate.GetUploadEraseFlash();

        if (progressBar1.Value == 100)
        {
          // Increment step to go to next command
          step++;

          // Change the text of the label
          labelUpdateState.Text = "Update: Programming ... "
                                  + fWUpdate.GetUploadEraseFlash().ToString() + "%";
        }
      }
      // Start the upload of the new firmware
      // stop/finish the firmware update
      /*step6*/
      else if (step == 6)
      {
        // Change the text of the label
        labelUpdateState.Text = "Update: Finished";

        // Stop the timer
        timerFWUpdate.Stop();

        // Reset the step
        step = 0;

        /*Close interface*/
        connClose();
        // Show a message box to inform that the 
        // firmware update has finished
        MessageBox.Show("Update finished." +
                       "\nThe visualizer restarts in 30 - 150 seconds.");
      }
    }
    //@@@STOP_STATEMACHINE_PF1@@@

    /*
     * Decide if upgrade or downgrade
     * @param iResult: +1 = upgrade
     *                  0 = quit
     *                 -1 = downgrade
     */
    public Boolean FWUpOrDowngradeControl(ref int iResult)
    {
      // Stop the timer
      timerFWUpdate.Stop();

      // Type of visualizer
      Byte[] type = new Byte[6];
      for (int i = fWData.Length - 30; i < fWData.Length - 24; i++)
      {
        type[i - (fWData.Length - 30)] = fWData[i];
      }

      // Buildnumber
      Byte[] buildnumber = new Byte[14];
      for (int i = fWData.Length - 24; i < fWData.Length - 10; i++)
      {
        buildnumber[i - (fWData.Length - 24)] = fWData[i];
      }

      // Version
      Byte[] version = new Byte[8];
      for (int i = fWData.Length - 10; i < fWData.Length - 2; i++)
      {
        version[i - (fWData.Length - 10)] = fWData[i];
      }

      // Convert the byte arrays into string to show them
      System.Text.Encoding enc = System.Text.Encoding.ASCII;
      string strType = enc.GetString(type);
      string strBuildnumber = enc.GetString(buildnumber);
      string strVersion = enc.GetString(version);

      // Initialize the command to get the actually buildnumber
      FirmwareUpdate fWUpdate = new FirmwareUpdate(comIf);
      /*Send command*/
      Byte[] oldBuildnumber = fWUpdate.GetBuildnumber();
      if (oldBuildnumber == null)
        return true;

      // Convert the buildnumber to string
      string strOldBuildnumber = enc.GetString(oldBuildnumber);

      /* Control if the new firmware is a newer or an older version
       * if it's an older one, it's a downgrade
       */
      if (Convert.ToInt64(strOldBuildnumber) <= Convert.ToInt64(strBuildnumber))
      {
        // Restart the timer event
        timerFWUpdate.Start();
        iResult = 1;
      }
      else
      {
        // Query if the user wants to downgrade, if not, stop the downgrade
        DialogResult dlgResult = MessageBox.Show("You are going to downgrade your firmware"
                                                 + "\n Type" + strType
                                                 + "\n Version" + strVersion
                                                 + "\n Do you really want to do?",
                                                 "Question", MessageBoxButtons.YesNo);
        if (dlgResult == DialogResult.Yes)
        {
          // Restart the timer
          timerFWUpdate.Start();
          iResult = -1;
        }
        else
        {
          iResult = 0;
        }
      }
      return false;
    }

    /**
     * Upload th FW file
     * @param strAction: show info in message box
     */
    public Boolean Upload(String strAction)
    {
      // Stop the timer
      timerFWUpdate.Stop();

      // Create a new integer variable
      int counter = 0;
      int dataLength = 0;

      // Reset the value of the progress bar
      progressBar1.Value = 0;

      /* Upload packets of the firmware until the whole 
       * firmware is uploaded
       */
      while (counter * maxPacketLength <= fWData.Length)
      {
        // Generate the length of upload packet
        if ((counter + 1) * maxPacketLength > fWData.Length)
          dataLength = fWData.Length - counter * maxPacketLength;
        else
          dataLength = maxPacketLength;

        // Generate the count of already sended bytes
        int alreadySended = counter * maxPacketLength;

        long lHelp = (long)alreadySended;
        // Generate the percentage of already uploaded bytes
        long uploadPercentage = 100 * lHelp / fWData.Length;

        // Change the text of the label
        labelUpdateState.Text = "Update: " + strAction + " ... " + uploadPercentage + "%";

        // Change the value of the progress bar
        progressBar1.Value = (int)uploadPercentage;

        // Enable other events outside of the loop
        Application.DoEvents();

        // Initialize the method to upload the new firmware
        FirmwareUpdate fWUpdate = new FirmwareUpdate(comIf);
        if (fWUpdate.UploadData(dataLength, alreadySended, fWData))
          return true;

        // Increment the counter
        counter++;
      }

      // Reset the progress 
      progressBar1.Value = 0;
      // Start the timer
      timerFWUpdate.Start();
      return false;
    }

    private void buttonUpdate_Click(object sender, EventArgs e)
    {
      // State machine
      step = 0;

      buttonReturn.Enabled = false;
      buttonUpdate.Enabled = false;
      buttonSearch.Enabled = false;

      fWData = readFwFile();

      /*Open interface*/
      if (connOpen())
        return;

      // Starts the timer to start the firmware Update state machine
      timerFWUpdate.Start();
    }

    private Byte[] readFwFile()
    {
      // Create a new filestream
      FileStream fs = new FileStream(textBoxPath.Text, FileMode.Open, FileAccess.Read);

      // Create a byte array for data of file stream 
      Byte[] buf = new Byte[fs.Length];

      // Read block of bytes from stream into the byte array  
      fs.Read(buf, 0, System.Convert.ToInt32(fs.Length));

      // Close the filestream
      fs.Close();
      return buf;
    }

    private void FormFirmwareUpdate_FormClosing(object sender, FormClosingEventArgs e)
    {
      timerFWUpdate.Stop();
      connClose();
    }

  }
}
