﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace VZInterfaceTestHandbook
{

  public partial class FormPictureTransfer : Form
  {
    IComIf comIf;
    bool continuousPicture = false;
    InterfaceType iInterface;
    Boolean isConnected = false;
    class ResTable
    {
      public String strResolution;
      public int iWidth;
      public int iHeight;
      public int iId;
    }
    System.Collections.ArrayList rt = new System.Collections.ArrayList();

    /*
     * @param com: uses Interface IComIf
     * @param itf: interface
     */
    public FormPictureTransfer(IComIf com, InterfaceType itf)
    {
      comIf = com;
      iInterface = itf;

      InitializeComponent();
    }

    /**
     * Open connection
     * @return success = false
     */
    private Boolean connOpen()
    {
      if (comIf.Open())
      {
        MessageBox.Show("Couldn´t open interface!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return true;
      }
      isConnected = true;
      return false;
    }

    /**
     * Close connection
     */
    private void connClose()
    {
      if (isConnected)
        comIf.Close();
      isConnected = false;
    }

    private void buttonReturn_Click(object sender, EventArgs e)
    {
      connClose();
      Close();
    }

    //@@@START_BUTTONGETPICTURE@@@
    private void buttonGetPicture_Click(object sender, EventArgs e)
    {
      if (buttonGetPicture.Text == "Get Picture")
      {
        disableControls(1);
        continuousPicture = true;
        if (connOpen())
          return;
        takeAPic();
        connClose();
        enableControls(1);
      }
      else
      {
        continuousPicture = false;
        buttonGetPicture.Text = "Get Picture";
      }
    }
    //@@@STOP_BUTTONGETPICTURE@@@

    private void buttonGetPictureV2_Click(object sender, EventArgs e)
    {
      if (buttonGetPictureV2.Text == "Get Picture V2")
      {
        disableControls(2);
        continuousPicture = true;
        if (connOpen())
          return;
        takeAPicV2();
        connClose();
        enableControls(2);
      }
      else
      {
        continuousPicture = false;
        buttonGetPictureV2.Text = "Get Picture V2";
      }
    }

    private void buttonSavePicture_Click(object sender, EventArgs e)
    {
      // Ensure that a picture is choosen
      if (pictureBox1.Image == null)
      {
        MessageBox.Show("Warning! First load a picture from your device", "Warning!",
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
        return;
      }

      SaveFileDialog saveFileDialog1 = new SaveFileDialog();
      saveFileDialog1.Filter = "JPeg Image|*.jpg|Bitmap Image|*.bmp";
      saveFileDialog1.Title = "Save Picture";

      // Open dialog and check filname, etc
      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
        string filename = saveFileDialog1.FileName;
        pictureBox1.Image.Save(filename); // Save picture of picturebox1
      }
    }

    /**
     * Get picture in legacy mode (1 byte ids)
     */
    private void takeAPic()
    {
      while (continuousPicture)
      {
        //@@@START_D0@@@
        int i;
        int intpicturesettings = 0;
        Boolean isBmp;

        // Motion Settings
        if (rBMotionContinuous.Checked == true)      // when Continuous is used
        {
          buttonGetPicture.Text = "STOP";
          //intpicturesettings |= 0x80;              // dont set the continuous flag, this flag is reserved for streaming
        }
        else
          continuousPicture = false;

        // Format Settings
        if (rBFormatJPG.Checked == true)             // when JPG is used
        {
          intpicturesettings |= 0x40;
          isBmp = false;
        }
        else                                         // when BMP is used
          isBmp = true;

        // Quality Settings
        if (rBQualityGood.Checked == true)           // when good quality is used
          intpicturesettings |= 10;
        else if (rBQualityAverage.Checked == true)   // when average quality is used
          intpicturesettings |= 20;
        else if (rBQualityLow.Checked == true)       // when low quality is used
          intpicturesettings |= 30;

        // Index setting
        i = comboBoxResTable.SelectedIndex;          // index of combobox is index of resolution table
        intpicturesettings |= i & 0x0F;

        // Converts the int number in a Byte
        Byte picturesettings = Convert.ToByte(intpicturesettings);
        //@@@STOP_D0@@@

        PictureTransfer pT = new PictureTransfer(comIf);
        Byte[] buf;
        if (iInterface == InterfaceType.ETH)
        {
          buf = pT.GetPictureETH(picturesettings, isBmp);
        }
        else
        {
          buf = pT.GetPictureUSB(picturesettings, isBmp);
        }
        if (buf == null)
          return;

        MemoryStream picStream;
        Image img;

        if (buf.Length != 3)
        {
          if (iInterface == InterfaceType.ETH)
            picStream = new MemoryStream(buf, 7, buf.Length - 7);
          else
            picStream = new MemoryStream(buf, 0, buf.Length - 4);
          img = Image.FromStream(picStream);
          pictureBox1.Image = img;
          pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
        }

        // Enable other events outside of the loop
        Application.DoEvents();
      }
    }

    /*
     * Get picture version 2 (4 byte IDs)
     */
    private void takeAPicV2()
    {
      while (continuousPicture)
      {
        //@@@START_D0V2@@@
        int i;
        int intpicturesettings = 0;
        int iId;
        ResTable clResTable = new ResTable();

        // Motion Settings
        if (rBMotionContinuous.Checked == true)      // when Continuous is used
        {
          buttonGetPictureV2.Text = "STOP";
          //intpicturesettings |= 0x80;              // dont set the continuous flag, reserved for streaming
        }
        else
          continuousPicture = false;

        // Format Settings
        if (rBFormatJPG.Checked == true)             // when JPG is used
        {
          intpicturesettings |= 0x40;
        }

        // Quality Settings
        if (rBQualityGood.Checked == true)           // when good quality is used
          intpicturesettings |= 10;
        else if (rBQualityAverage.Checked == true)   // when average quality is used
          intpicturesettings |= 20;
        else if (rBQualityLow.Checked == true)       // when low quality is used
          intpicturesettings |= 30;

        // Index setting
        i = comboBoxResTable.SelectedIndex;          // index of combobox
        clResTable = (ResTable)rt[i];
        iId = clResTable.iId;

        // Converts the int number in a Byte
        Byte picturesettings = Convert.ToByte(intpicturesettings);
        //@@@STOP_D0V2@@@

        PictureTransfer pT = new PictureTransfer(comIf);
        Byte[] buf;
        buf = pT.GetPictureV2(picturesettings, iId);
        if (buf == null)
          return;

        MemoryStream picStream;
        Image img;

        if (buf.Length != 3)
        {
          picStream = new MemoryStream(buf, 7, buf.Length - 7);
          img = Image.FromStream(picStream);
          pictureBox1.Image = img;
          pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
        }

        // Enable other events outside of the loop
        Application.DoEvents();
      }
    }

    private void buttonMultiUnicast_Click(object sender, EventArgs e)
    {
      FormMultiUnicast fMU = new FormMultiUnicast(comIf);
      fMU.Show();
    }

    private void buttonGetResTable_Click(object sender, EventArgs e)
    {
      Byte[] bufRecv;
      /*Instantiate class*/
      PictureTransfer pT = new PictureTransfer(comIf);
      /*Open interface*/
      if (connOpen())
        return;
      /*Send command*/
      bufRecv = pT.GetResTable();
      fillResTable(bufRecv);
      comboBoxResTable.SelectedIndex = 0;
      /*Close interface*/
      connClose();
      buttonSavePicture.Enabled = true;
      buttonGetPicture.Enabled = true;
      buttonGetPictureV2.Enabled = true;
    }

    /**
     * Fill the resolution table structure
     */
    private void fillResTable(Byte[] buf)
    {
      int i;
      int iLen;

      if ((buf[0] & Wolfprot2.HEADER_EXT_LEN) == Wolfprot2.HEADER_EXT_LEN)   //Resolution Table new
      {
        iLen = (buf[2] << 8) | buf[3];
        for (i = 4; i < iLen; i += 72)
        {
          ResTable clResTable = new ResTable();
          String strResolution = System.Text.Encoding.ASCII.GetString(buf).Substring(i, 63); //cut off string end 0
          strResolution = strResolution.Trim();
          clResTable.strResolution = strResolution;
          int iWidth = (buf[i + 64] << 8) | buf[i + 64 + 1];
          clResTable.iWidth = iWidth;
          int iHeight = (buf[i + 66] << 8) | buf[i + 66 + 1];
          clResTable.iHeight = iHeight;
          int iId = (buf[i + 68] << 24) | (buf[i + 68 + 1] << 16) | (buf[i + 68 + 2] << 8) | buf[i + 68 + 3];
          clResTable.iId = iId;
          rt.Add(clResTable);
          String strItem = strResolution + " (" + iWidth + "x" + iHeight + ")";
          comboBoxResTable.Items.Add(strItem);
        }
      }
      else                                               //Resolution Table legacy
      {
        iLen = buf[2];
        for (i = 3; i < iLen; i += 14)
        {
          ResTable clResTable = new ResTable();
          String strResolution = System.Text.Encoding.ASCII.GetString(buf).Substring(i, 7); //cut off string end 0
          strResolution = strResolution.Trim();
          clResTable.strResolution = strResolution;
          int iWidth = (buf[i + 8] << 8) | buf[i + 8 + 1];
          clResTable.iWidth = iWidth;
          int iHeight = (buf[i + 10] << 8) | buf[i + 10 + 1];
          clResTable.iHeight = iHeight;
          int iId = (buf[i + 12] << 8) | buf[i + 12 + 1];
          clResTable.iId = iId;
          rt.Add(clResTable);
          String strItem = strResolution + " (" + iWidth + "x" + iHeight + ")";
          comboBoxResTable.Items.Add(strItem);
        }
      }
    }

    private void enableControls(int vPic)
    {
      if (vPic == 1)
        buttonGetPictureV2.Enabled = true;
      else
        buttonGetPicture.Enabled = true;
      buttonGetResTable.Enabled = true;
      buttonReturn.Enabled = true;
      buttonMultiUnicast.Enabled = true;
      buttonSavePicture.Enabled = true;
    }

    private void disableControls(int vPic)
    {
      if (vPic == 1)
        buttonGetPictureV2.Enabled = false;
      else
        buttonGetPicture.Enabled = false;
      buttonGetResTable.Enabled = false;
      buttonReturn.Enabled = false;
      buttonMultiUnicast.Enabled = false;
      buttonSavePicture.Enabled = false;
    }

    private void FormPictureTransfer_FormClosing(object sender, FormClosingEventArgs e)
    {
      connClose();
    }
  }
}
