﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using VZInterfaceTestHandbook;

namespace VZInterfaceTestHandbook
{
  /**
   * Main Form
   */
  public partial class Handbook : Form
  {
    public Handbook()
    {
      InitializeComponent();
      updateWolprotElements(true);
    }

    /**
    * @param bInit: Clears the cbComPorts items and search for new when initializing
    */
    private void updateWolprotElements(Boolean bInit)
    {
      if (bInit)
      {
        try
        {
          cbComPorts.Items.Clear();
          foreach (string s in System.IO.Ports.SerialPort.GetPortNames())             // Search for the Com ports (needed for RS232)
          {
            cbComPorts.Items.Add(s);
          }

          cbComPorts.SelectedIndex = 1;                                               // Selected Com port, when programm starts (COM1)
        }                                                                             
        catch (Exception)
        { }
      }
      if (rbUSB.Checked)                                                                  
      {
        cbComPorts.Enabled = false;
        tbIp.Enabled = false;
        buttonScanUSBDevices.Enabled = true;
        cbUSbDevices.Enabled = true;
      }
      else if (rbETH.Checked)                                                             
      {
        cbComPorts.Enabled = false;
        tbIp.Enabled = true;
        buttonScanUSBDevices.Enabled = false;
        cbUSbDevices.Enabled = false;
      }
      else                                                                                
      {
        cbComPorts.Enabled = true;
        tbIp.Enabled = false;
        buttonScanUSBDevices.Enabled = false;
        cbUSbDevices.Enabled = false;
      }
    }
    //@@@START_SELECTEDINTERFACE@@@
    public IComIf SelectedInterface()
    {
      IComIf comIf;
      RS232Com com232;
      EthCom comETH;
      UsbCom comUSB;

      if (rbUSB.Checked)                                                  // when radiobutton USB is used
      {
        comUSB = new UsbCom();
        comIf = comUSB;
      }
      else if (rbETH.Checked)                                             // when radiobutton ETH is used
      {
        comETH = new EthCom();
        comETH.Configure(tbIp.Text, 50915);
        comIf = comETH;
      }
      else                                                                // when radiobutton RS232 is used
      {
        com232 = new RS232Com();
        com232.Configure(cbComPorts.SelectedItem.ToString(), 115200);
        comIf = com232;
      }
      return comIf;
    }
    //@@@STOP_SELECTEDINTERFACE@@@

    private void rbRS232_CheckedChanged(object sender, EventArgs e)
    {
      updateWolprotElements(false);
    }

    private void rbETH_CheckedChanged(object sender, EventArgs e)
    {
      updateWolprotElements(false);
    }

    private void rbUSB_CheckedChanged(object sender, EventArgs e)
    {
      updateWolprotElements(false);
    }

    //@@@START_BUTTONSCANUSBDEVICES_CLICK@@@
    private void buttonScanUSBDevices_Click(object sender, EventArgs e)
    {
      uint uiDeviceCount;
      int i = 0;


      cbUSbDevices.Items.Clear();

      uiDeviceCount = USBManager.USBFindAllDevs();
      if (uiDeviceCount == 0)
      {
        MessageBox.Show("No Visualizer found on USB", "No Visualizer found", MessageBoxButtons.OK, MessageBoxIcon.Stop);
        return;
      }

      foreach (WinUsbNet.UsbDevice USBDevice in USBManager.USBMan.UsbDevices)
      {
        Byte[] baSerial = new Byte[100];

        if (USBDevice.IsOpen) USBDevice.Close();
        USBManager.currUSBDevice = USBDevice;
        try
        { 
          if (!USBDevice.IsOpen)
            USBDevice.Open(); 
        }
        catch (Exception)
        {
          MessageBox.Show("USB device " + Environment.NewLine + USBDevice.DeviceName + Environment.NewLine + "already opened by another application (probably WolfVision Connectivity Software 2)" + Environment.NewLine + "Please close this application to grant access to the USB device", "USB Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
          continue;
        }

        i += 1;
        USBDevice.Tag = USBManager.GetTypeSerialVersion(USBDevice);
        string[] DeviceInfo = USBDevice.Tag.ToString().Split(new char[] { ';' });

        cbUSbDevices.Items.Add(i + " - " + DeviceInfo[0] + " (" + DeviceInfo[1] + ") " + DeviceInfo[2]);

      }

      if (cbUSbDevices.Items.Count > 0)
        cbUSbDevices.SelectedIndex = 0;

    }
    //@@@STOP_BUTTONSCANUSBDEVICES_CLICK@@@

    private void buttonPowerSettings_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormPowerSettings fPS = new FormPowerSettings(comIf);
      fPS.Show();
    }

    private void buttonZoomSettings_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormZoomSettings fZS = new FormZoomSettings(comIf);
      fZS.Show();
    }

    private void buttonPictureTransfer_Click(object sender, EventArgs e)
    {
      InterfaceType iInterface;

      iInterface = InterfaceType.RS232;
      if (rbETH.Checked)
        iInterface = InterfaceType.ETH;
      if (rbUSB.Checked)
        iInterface = InterfaceType.USB;
      if (rbETH.Checked || rbUSB.Checked)
      {
        IComIf comIf = SelectedInterface();
        FormPictureTransfer fPT = new FormPictureTransfer(comIf, iInterface);
        fPT.Show();
      }
      else
      {
        MessageBox.Show("ETH or USB must be used!", "Error", MessageBoxButtons.OKCancel);
      }
    }

    private void buttonAuthentication_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormAuthentication fa = new FormAuthentication(comIf);
      fa.Show();
    }

    /**
    * Get IP address from editbox
    */
    public string sendIP()
    {
      return tbIp.Text;
    }

    private void buttonXMLFile_Click(object sender, EventArgs e)
    {
      if (rbETH.Checked)
      {
        IComIf comIf = SelectedInterface();
        FormXMLFile fxml = new FormXMLFile(comIf);
        fxml.Show();
      }
      else
        MessageBox.Show("ETH must be used!", "Error", MessageBoxButtons.OK);
    }

    private void buttonHeightAdj_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormHeightAdjustment fHeightadj = new FormHeightAdjustment(comIf);
      fHeightadj.Show();
    }

    private void buttonSearchDevices_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormDevices fDevices = new FormDevices(comIf);
      fDevices.Show();
    }

    private void buttonFirmwareUpdate_Click(object sender, EventArgs e)
    {
      int iPacketLen = new int();

      if (rbRS232.Checked)
        MessageBox.Show("Not avaible for RS232!", "Error", MessageBoxButtons.OK);
      else
      {
        if (rbUSB.Checked)
        {
          iPacketLen = 508;
        }
        else
        {
          iPacketLen = 1440;
        }

        IComIf comIf = SelectedInterface();
        FormFirmwareUpdate fFirmwareUpdate = new FormFirmwareUpdate(comIf, iPacketLen);
        fFirmwareUpdate.Show();
      }
    }

    private void cbUSbDevices_SelectedIndexChanged(object sender, EventArgs e)
    {
      ComboBox cbo = (ComboBox)sender;
      string[] serial = cbo.SelectedItem.ToString().Split(new char[] { '(', ')' });
      if (serial.Length.Equals(3))
      {
        USBManager.currUSBDevice = (WinUsbNet.UsbDevice)(from WinUsbNet.UsbDevice usbdev in USBManager.USBMan.UsbDevices
                                                         where usbdev.Tag != null && usbdev.Tag.ToString().Contains(serial[1])
                                                         select usbdev).FirstOrDefault();
      }
    }

    private void buttonBlockCommands_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormBlockCommands frmBlkCmds = new FormBlockCommands(comIf);
      frmBlkCmds.Show();
    }

    private void buttonAnyCommand_Click(object sender, EventArgs e)
    {
      IComIf comIf = SelectedInterface();
      FormAnyCommand frmAnyCommand = new FormAnyCommand(comIf);
      frmAnyCommand.Show();
    }
  }
}
