﻿
using System;
using System.Windows;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing;

namespace VZInterfaceTestHandbook
{
  class PictureTransfer
  {
    IComIf comIf;
    /**
     * @param com: uses Interface IComIf
     */
    public PictureTransfer(IComIf com)
    {
      comIf = com;
    }

    //@@@START_PICTUREHEADER@@@
    public Byte[] PictureHeader(Byte picturesettings)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;
      /*Declaration*/
      Byte[] bufSnd = new Byte[4];
      Byte[] bufRecv = new Byte[23];

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xB8;
      bufSnd[2] = 0x01;
      bufSnd[3] = picturesettings; // picturesettings get choosen in FormPictureSettings

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return null;
      }

      return bufRecv;
    }
    //@@@STOP_PICTUREHEADER@@@

    //@@@START_PICTUREHEADERV2@@@
    public Byte[] PictureHeaderV2(Byte picturesettings, int iId)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;
      /*Declaration*/
      Byte[] bufSnd = new Byte[8];
      Byte[] bufRecv = new Byte[23];

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xB8;
      bufSnd[2] = 0x05;
      bufSnd[3] = picturesettings;
      bufSnd[4] = Convert.ToByte((iId & 0xFF000000) >> 24);
      bufSnd[5] = Convert.ToByte((iId & 0x00FF0000) >> 16);
      bufSnd[6] = Convert.ToByte((iId & 0x0000FF00) >> 8);
      bufSnd[7] = Convert.ToByte(iId & 0x000000FF);

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return null;
      }

      return bufRecv;
    }
    //@@@STOP_PICTUREHEADERV2@@@

    //@@@START_GETPICTUREBLOCKETH@@@
    public Byte[] GetPictureBlockETH(Byte[] pic_header, Boolean isBmp)
    {
      int iWidth, iHeight;
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;
      /*Declaration*/
      Byte[] bufSnd = new Byte[25];
      Byte[] bufRecv = new Byte[6220800 + 54 + 7]; //protcolheader + bmpheader + bmpcontents (1920x1080x3)

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xBA;
      bufSnd[2] = 0x16;

      // Flags
      bufSnd[3] = pic_header[7];

      // Mode Byte
      bufSnd[4] = pic_header[8];

      // Offset
      bufSnd[5] = 0x00;
      bufSnd[6] = 0x00;
      bufSnd[7] = 0x00;
      bufSnd[8] = 0x00;

      // Max Blocklength
      int blocklength;

      blocklength = Convert.ToInt32(pic_header[15] << 24)
                  + Convert.ToInt32(pic_header[16] << 16)
                  + Convert.ToInt32(pic_header[17] << 8)
                  + Convert.ToInt32(pic_header[18]);

      if (blocklength == 0)
        return null;

      iWidth = Convert.ToInt32(pic_header[3] << 8) + Convert.ToInt32(pic_header[4]);
      iHeight = Convert.ToInt32(pic_header[5] << 8) + Convert.ToInt32(pic_header[6]);

      // If legacy mode length is without headerlen
      if (isBmp)
      {
        if (blocklength == (iWidth * iHeight * 3))
          blocklength += 54;
      }
      bufSnd[9] = Convert.ToByte((blocklength & 0xFF000000) >> 24);
      bufSnd[10] = Convert.ToByte((blocklength & 0x00FF0000) >> 16);
      bufSnd[11] = Convert.ToByte((blocklength & 0x0000FF00) >> 8);
      bufSnd[12] = Convert.ToByte(blocklength & 0x000000FF);

      // Width
      bufSnd[13] = pic_header[3];
      bufSnd[14] = pic_header[4];

      // Height
      bufSnd[15] = pic_header[5];
      bufSnd[16] = pic_header[6];

      // Total Size
      bufSnd[17] = pic_header[9];
      bufSnd[18] = pic_header[10];
      bufSnd[19] = pic_header[11];
      bufSnd[20] = pic_header[12];

      // Memory Page
      bufSnd[21] = pic_header[19];
      bufSnd[22] = pic_header[20];

      // Out Offset
      bufSnd[23] = pic_header[21];
      bufSnd[24] = pic_header[22];

      /*Send command 0xBA*/
      iReturn = wp2.execCmdFast(bufSnd, ref bufRecv, blocklength + 7); //7 bytes Wolfprot protocol

      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return null;
      }

      return bufRecv;
    }
    //@@@STOP_GETPICTUREBLOCKETH@@@

    //@@@START_GETPICTUREETH@@@
    public Byte[] GetPictureETH(Byte picturesettings, Boolean isBmp)
    {
      /*Declaration*/
      Byte[] bufRecv;
      Byte[] b;

      /*Send command 0xB8*/
      b = PictureHeader(picturesettings); // use method PictureHeader
      if (b == null)
        return null;

      /*Send command 0xBA*/
      bufRecv = GetPictureBlockETH(b, isBmp);
      if (bufRecv == null)
        return (null);
      return (bufRecv);
    }
    //@@@STOP_GETPICTUREETH@@@

    /**
     * get a USB picture block.
     * Note: B9 commands have the Wolfprot prefix at the end of the message block instead of the beginning.
     * This is due to the USB protocol using different pipes for Wolfprot and image.
     */
    public Byte[] GetPictureBlockUSB(Byte[] pic_header)
    {
      int iWidth, iHeight;
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;
      /*Declaration*/
      Byte[] bufSnd = new Byte[13];
      Byte[] bufRecv = new Byte[6220800 + 54 + 4]; // bmp + header + cmd acknowledge

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xB9;
      bufSnd[2] = 0x0A;

      // Flags
      bufSnd[3] = pic_header[7];

      // Mode Byte
      bufSnd[4] = pic_header[8];

      // Offset
      bufSnd[5] = 0x00;
      bufSnd[6] = 0x00;
      bufSnd[7] = 0x00;
      bufSnd[8] = 0x00;

      // Max blocklength
      int blocklength;

      blocklength = Convert.ToInt32(pic_header[15] << 24)
                  + Convert.ToInt32(pic_header[16] << 16)
                  + Convert.ToInt32(pic_header[17] << 8)
                  + Convert.ToInt32(pic_header[18]);

      if (blocklength == 0)
        return null;

      iWidth = Convert.ToInt32(pic_header[3] << 8) + Convert.ToInt32(pic_header[4]);
      iHeight = Convert.ToInt32(pic_header[5] << 8) + Convert.ToInt32(pic_header[6]);

      bufSnd[9] = Convert.ToByte((blocklength & 0xFF000000) >> 24);
      bufSnd[10] = Convert.ToByte((blocklength & 0x00FF0000) >> 16);
      bufSnd[11] = Convert.ToByte((blocklength & 0x0000FF00) >> 8);
      bufSnd[12] = Convert.ToByte(blocklength & 0x000000FF);


      /*Send command 0xB9*/
      iReturn = wp2.execCmdFast(bufSnd, ref bufRecv, blocklength + 4);

      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else if (((bufRecv[0] & 0x80) == 0x80) && (bufRecv.Length == 3))  //only the error message uses wolfprot
      {
        wp2.ErrorSummary(bufRecv[2]);
        return null;
      }

      return bufRecv;
    }

    /**
     * BMP header
     * Be aware that the data format is little endian
     */
    private Byte[] createBmpHeader(Byte[] pic_header)
    {
      int iWidth, iHeight;
      Byte[] bmp_header = new Byte[54];

      iWidth = Convert.ToInt32(pic_header[3] << 8) + Convert.ToInt32(pic_header[4]);
      iHeight = Convert.ToInt32(pic_header[5] << 8) + Convert.ToInt32(pic_header[6]);

      bmp_header[0] = Convert.ToByte('B');
      bmp_header[1] = Convert.ToByte('M');
      int bfSize = iHeight * iWidth * 3 + 54;
      fillInt(ref bmp_header, 2, bfSize);
      short bfReserved1 = 0;
      fillShort(ref bmp_header, 6, bfReserved1);
      short bfReserved2 = 0;
      fillShort(ref bmp_header, 8, bfReserved2);
      int bfOffBits = 54;        // Offset für RGB Daten Start
      fillInt(ref bmp_header, 10, bfOffBits);
      int biSize = 40;        // Size of Infoheader
      fillInt(ref bmp_header, 14, biSize);
      int biWidth = iWidth;
      fillInt(ref bmp_header, 18, biWidth);
      int biHeight = iHeight;
      fillInt(ref bmp_header, 22, biHeight);
      short biPlanes = 1;
      fillShort(ref bmp_header, 26, biPlanes);
      short biBitCount = 24;
      fillShort(ref bmp_header, 28, biBitCount);
      int biCompression = 0;
      fillInt(ref bmp_header, 30, biCompression);
      int biSizeImage = iHeight * iWidth * 3;
      fillInt(ref bmp_header, 34, biSizeImage);
      int biXPelsPerMeter = 0;
      fillInt(ref bmp_header, 38, biXPelsPerMeter);
      int biYPelsPerMeter = 0;
      fillInt(ref bmp_header, 42, biYPelsPerMeter);
      int biClrUsed = 0;
      fillInt(ref bmp_header, 46, biClrUsed);
      int biClrImportant = 0;
      fillInt(ref bmp_header, 50, biClrImportant);

      return bmp_header;
    }

    /**
     * Fill in int little endian
     */
    public void fillInt(ref Byte[] buf, int iIdx, int iVal)
    {
      buf[iIdx + 3] = Convert.ToByte((iVal & 0xFF000000) >> 24);
      buf[iIdx + 2] = Convert.ToByte((iVal & 0x00FF0000) >> 16);
      buf[iIdx + 1] = Convert.ToByte((iVal & 0x0000FF00) >> 8);
      buf[iIdx] = Convert.ToByte((iVal & 0x000000FF) >> 0);
    }

    /**
     * Fill in short little endian
     */
    public void fillShort(ref Byte[] buf, int iIdx, short sVal)
    {
      buf[iIdx + 1] = Convert.ToByte((sVal & 0x0000FF00) >> 8);
      buf[iIdx] = Convert.ToByte((sVal & 0x000000FF) >> 0);
    }

    public Byte[] GetPictureUSB(Byte picturesettings, Boolean isBmp)
    {
      /*Declaration*/
      Byte[] bufRecv;
      Byte[] b;

      /*Send command 0xB8*/
      b = PictureHeader(picturesettings); // use method PictureHeader
      if (b == null)
        return (null);

      bufRecv = GetPictureBlockUSB(b);
      if (bufRecv == null)
        return (null);
      if (isBmp)
      {
        // Add BMP header, turn array upsidedown
        Byte[] bmpheader = createBmpHeader(b);
        Byte[] copy = new Byte[bufRecv.Length];
        int iWidth = Convert.ToInt32(b[3] << 8) + Convert.ToInt32(b[4]);
        int iHeight = Convert.ToInt32(b[5] << 8) + Convert.ToInt32(b[6]);
        for (int i = 0; i < iHeight; i++)
        {
          System.Array.Copy(bufRecv, ((iHeight - 1) - i) * (iWidth * 3), copy, i * (iWidth * 3), iWidth * 3);
        }
        byte[] concat = new byte[bufRecv.Length];
        System.Array.Copy(bmpheader, 0, concat, 0, bmpheader.Length);
        System.Array.Copy(copy, 0, concat, 54, copy.Length - 54);
        System.Array.Copy(concat, 0, bufRecv, 0, bufRecv.Length);
      }
      return bufRecv;
    }

    //@@@START_GETPICTUREBLOCKV2@@@
    public Byte[] GetPictureBlockV2(Byte[] pic_header, int iId)
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;

      /*Declaration*/
      Byte[] bufSnd = new Byte[25];
      Byte[] bufRecv = new Byte[6220800 + 54 + 7]; //bmp + header

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0xBA;
      bufSnd[2] = 0x16;

      // Flags
      bufSnd[3] = pic_header[7];

      // Mode Byte
      bufSnd[4] = pic_header[8];

      // Offset
      bufSnd[5] = 0x00;
      bufSnd[6] = 0x00;
      bufSnd[7] = 0x00;
      bufSnd[8] = 0x00;

      // Max blocklength
      int blocklength;

      blocklength = Convert.ToInt32(pic_header[15] << 24)
                  + Convert.ToInt32(pic_header[16] << 16)
                  + Convert.ToInt32(pic_header[17] << 8)
                  + Convert.ToInt32(pic_header[18]);
      if (blocklength == 0)
        return null;


      bufSnd[9] = Convert.ToByte((blocklength & 0xFF000000) >> 24);
      bufSnd[10] = Convert.ToByte((blocklength & 0x00FF0000) >> 16);
      bufSnd[11] = Convert.ToByte((blocklength & 0x0000FF00) >> 8);
      bufSnd[12] = Convert.ToByte(blocklength & 0x000000FF);

      // Width
      bufSnd[13] = pic_header[3];
      bufSnd[14] = pic_header[4];

      // Height
      bufSnd[15] = pic_header[5];
      bufSnd[16] = pic_header[6];

      // Total Size
      bufSnd[17] = pic_header[15];
      bufSnd[18] = pic_header[16];
      bufSnd[19] = pic_header[17];
      bufSnd[20] = pic_header[18];

      // Memory Page
      bufSnd[21] = pic_header[19];
      bufSnd[22] = pic_header[20];

      // Out Offset
      bufSnd[23] = pic_header[21];
      bufSnd[24] = pic_header[22];

      /*Send command 0xBA*/
      iReturn = wp2.execCmdFast(bufSnd, ref bufRecv, blocklength + 7);
      //iReturn = wp2.execCmd(bufSnd, ref bufRecv);

      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else if ((bufRecv[0] & 0x80) == 0x80)
      {
        wp2.ErrorSummary(bufRecv[2]);
        return null;
      }

      return bufRecv;
    }
    //@@@STOP_GETPICTUREBLOCKV2@@@

    //@@@START_GETPICTUREV2@@@
    public Byte[] GetPictureV2(Byte picturesettings, int iId)
    {
      /*Declaration*/
      Byte[] bufRecv;
      Byte[] b;

      /*Send command 0xB8*/
      b = PictureHeaderV2(picturesettings, iId); // use method PictureHeader
      if (b == null)
        return null;

      bufRecv = GetPictureBlockV2(b, iId);
      if (bufRecv == null)
        return null;

      return bufRecv;
    }
    //@@@STOP_GETPICTUREV2@@@

    public Byte[] GetResTable()
    {
      Wolfprot2 wp2 = new Wolfprot2(comIf);
      Boolean iReturn;
      /*Declaration*/
      Byte[] bufSnd = new Byte[3];
      Byte[] bufRecv = new Byte[0xFFFF + 4];

      /*Assignment*/
      bufSnd[0] = 0x00;
      bufSnd[1] = 0x17;
      bufSnd[2] = 0x00;

      /*Send command*/
      iReturn = wp2.execCmd(bufSnd, ref bufRecv);
      if (iReturn == true)
      {
        MessageBox.Show("No communication!", "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
        return null;
      }
      else
      {
        if ((bufRecv[0] & 0x80) == 0x80)
        {
          wp2.ErrorSummary(bufRecv[2]);
          return null;
        }
        else
        {
          return (bufRecv);
        }
      }
    }

  }
}
