using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Collections;

namespace VZInterfaceTestHandbook
{
  public partial class UsbCom : IComIf
  {
    private WinUsbNet.UsbDevice usbDevice;

    private void DebugMsg(string strMSg)
    {
      //System.Console.WriteLine("USBCom: " + strMSg);
    }

    /**
     * Open the connection
     */
    public Boolean Open()
    {
      try
      {
        usbDevice = USBManager.currUSBDevice;

        if (!usbDevice.IsOpen)
          usbDevice.Open();
      }
      catch (Exception)
      {
      }
      return !usbDevice.IsOpen;
    }

    /**
     * Close the connection
     */
    public Boolean Close()
    {
      try
      {
        if (isConnected())
        {
          USBManager.currUSBDevice.Close();
        }
      }
      catch (Exception)
      {
      }
      return false;
    }

    public void Configure(WinUsbNet.UsbDevice usbDev)
    {
      this.usbDevice = usbDev;
    }

    public InterfaceType GetInterfaceType()
    {
      return InterfaceType.USB;
    }

    public Boolean SetBaudrate(int baudRate)
    {
      return false;
    }

    public Boolean isConnected()
    {
      return USBManager.currUSBDevice.IsOpen;
    }

    /**
     * Write bytes
     * @param buffer: bytes to write
     * @param count: number of bytes
     * @return: success = false
     */
    public Boolean Write(Byte[] buffer, int count)
    {
      Boolean bError = false;

      try
      {
        USBManager.SendPipe.Write(buffer, 0, count);
        //DebugMsg("Wrote " + count + " bytes (" + Toolbox.ByteArryToString(buffer, count) + ")");
      }
      catch (Exception)
      {
        return true;
      }

      return bError;
    }

    /**
     * Set the timeout
     * @param int timeout: time in ms
     */
    public Boolean ReadTimeout(int timeout)
    {
      USBManager.ReceivePipe.ReadTimeout = timeout;
      return false;
    }

    /**
     * Available bytes
     * return 0 because not implemented
     */
    public int BytesToRead()
    {
      return 0;
    }

    public void Flush()
    {
    }

    /**
     * Read bytes currently available
     */
    public int Read(ref Byte[] buffer)
    {
      if (!isConnected())
        return -1;
      USBManager.ReceivePipe.Read(buffer, 0, buffer.Length);
      return buffer.Length;
    }

    /**
     * Read bytes currently available
     * @param buffer: to be filled
     * @param count: number of bytes to be read
     * return read bytes
     */
    public int Read(out byte[] buffer, int count)
    {
      Byte[] buf = new Byte[count];
      int iRead;

      if (!isConnected())
      {
        buffer = null;
        return -1;
      }
      USBManager.ReceivePipe.ReadTimeout = 1000;
      iRead = USBManager.ReceivePipe.Read(buf, 0, count);

      buffer = buf;
      return iRead;
    }

    /**
     * Read a single byte
     */
    public Boolean ReadByte(ref Byte byteRecv)
    {
      Boolean bError = false;
      if (!isConnected())
        return true;

      USBManager.ReceivePipe.ReadTimeout = 1000;
      byteRecv = (Byte)USBManager.ReceivePipe.ReadByte();

      return bError;
    }

    #region IComIf Members

    #endregion
  }
}
